import { IReadonlyObservableArray, IReadonlyObservableValue } from '../../Core/Observable';
import { ITabGroup, IVssContributedTab } from '../../Tabs';
import * as React from "react";
export declare enum Orientation {
    Vertical = 0,
    Horizontal = 1
}
/**
 * Properties for the Page component.
 */
export interface IPageProps {
    /**
     * Optional root classname for the page element.
     */
    className?: string;
    /**
     * Describes if the page should render elements in a flex-row
     * or flex-column
     */
    orientation?: Orientation;
    /**
     * Optional ref to the scrollable container for the page.
     */
    scrollableContainerRef?: React.RefObject<HTMLDivElement>;
    /**
     * Tracks the selected tab id within the page.
     */
    selectedTabId?: IReadonlyObservableValue<string> | string;
    /**
     * A list of providers contributing tab groups to the page.
     */
    tabGroups?: IReadonlyObservableArray<ITabGroup>;
    /**
     * A list of providers contributing tabs to the page.
     */
    tabProviders?: IReadonlyObservableArray<IVssContributedTab>;
    /**
     * Optional callback triggered when page's scroll event occurs.
     */
    onScroll?: React.UIEventHandler<HTMLDivElement>;
}
