import * as React from "react";
import { IDragDataTransfer } from '../Utilities/DragDrop';
import { ObservableValue } from '../Core/Observable';
/**
 * Exposes the x/y coordinates of the current drag/drop operation.
 * Used to determine where to draw the drag image.
 */
export interface IDragDropOperationOptimized {
    /**
     * X offset of current drag/drop operation
     */
    x: number | undefined;
    /**
     * Y offset of current drag/drop operation
     */
    y: number | undefined;
}
export declare function beginDragOperationOptimized<D, S>(event: React.PointerEvent<HTMLElement>, dataTransfer: IDragDataTransfer<D, S>, minimumPixelsForDrag?: number): ObservableValue<IDragDropOperationOptimized> | undefined;
export interface IDragItemPropsOptimized {
    /**
     * Optional class name to place on the content of the portal - the first element
     *  the consumer can control.
     */
    className?: string;
    /**
     * The drag/drop operation which has the current x/y values for the pointer position
     */
    operation: ObservableValue<IDragDropOperationOptimized>;
    /**
     * Number of pixels to offset the image from the pointer position horizontally.
     * @default 5
     */
    xOffset?: number;
    /**
     * Number of pixels to offset the image from the pointer position vertically.
     * @default 5
     */
    yOffset?: number;
}
export declare const DragImageOptimized: React.MemoExoticComponent<(props: IDragItemPropsOptimized & Readonly<{
    children?: React.ReactNode;
}>) => JSX.Element>;
